@@ -488,6 +488,9 @@ backend_map_config_filter(const char *format,
 		}
 	}
 	ret = malloc(l + 1);
+	if (ret == NULL) {
+		return NULL;
+	}
 	for (i = j = 0; format[i] != '\0'; i++) {
 		if (format[i] == '%') {
 			switch (format[i + 1]) {
@@ -1043,12 +1046,103 @@ backend_entry_is_delete_related(const char *group, const char *set, bool_t flag,
 {
 	return FALSE;
 }
+/* This task is called when the following entry is added
+ *
+ * dn: cn=rebuild 0,cn=NIS Server refresh task,cn=tasks,cn=config
+ * objectClass: top
+ * objectClass: extensibleObject
+ * cn: rebuild 0
+ */
+static int
+backend_nis_refresh_task(Slapi_PBlock *pb,
+                 Slapi_Entry *e,
+                 Slapi_Entry *eAfter __attribute__((unused)),
+                 int *returncode,
+                 char *returntext __attribute__((unused)),
+                 void *arg)
+{
+    PRThread *thread = NULL;
+    int rv = SLAPI_DSE_CALLBACK_OK;
+    Slapi_Task *task = NULL;
+    task_data *mytaskdata = NULL;
+    char *bind_dn;
+    const char *filter;
 
+    *returncode = LDAP_SUCCESS;
+
+    /* get the requestor dn for our thread data*/
+    slapi_pblock_get(pb, SLAPI_REQUESTOR_DN, &bind_dn);
+
+    /* default filter for mapped entries */
+    if ((filter = slapi_fetch_attr(e, "filter", NIS_MAP_CONFIGURATION_FILTER)) == NULL) {
+        *returncode = LDAP_OBJECT_CLASS_VIOLATION;
+        rv = SLAPI_DSE_CALLBACK_ERROR;
+        goto out;
+    }
+
+    /* allocate new task now */
+    task = slapi_plugin_new_task(slapi_entry_get_ndn(e), arg);
+    if (task == NULL) {
+        slapi_log_error(SLAPI_LOG_INFO, "nis-server", "backend_nis_refresh_task - Unable to allocate new task!\n");
+        *returncode = LDAP_OPERATIONS_ERROR;
+        rv = SLAPI_DSE_CALLBACK_ERROR;
+        goto out;
+    }
+
+    mytaskdata = (task_data *)slapi_ch_malloc(sizeof(task_data));
+    if (mytaskdata == NULL) {
+        *returncode = LDAP_OPERATIONS_ERROR;
+        rv = SLAPI_DSE_CALLBACK_ERROR;
+        goto out;
+    }
+    mytaskdata->state = nis_get_plugin_state();
+    mytaskdata->bind_dn = slapi_ch_strdup(bind_dn);
+    if (mytaskdata->bind_dn == NULL) {
+        slapi_log_error(SLAPI_LOG_INFO, "nis-server", "backend_nis_refresh_task - Unable initialize task data bind_dn!\n");
+        *returncode = LDAP_OPERATIONS_ERROR;
+        rv = SLAPI_DSE_CALLBACK_ERROR;
+        goto out;
+    }
+    mytaskdata->filter = slapi_ch_strdup(filter);
+    if (mytaskdata->filter == NULL) {
+        slapi_log_error(SLAPI_LOG_INFO, "nis-server", "backend_nis_refresh_task - Unable initialize task data filter!\n");
+        slapi_ch_free_string(&mytaskdata->bind_dn);
+        *returncode = LDAP_OPERATIONS_ERROR;
+        rv = SLAPI_DSE_CALLBACK_ERROR;
+        goto out;
+    }
+    mytaskdata->plugin_log_id = "nis-server";
+
+    /* set a destructor that will clean up schemadir for us when the task is complete */
+    slapi_task_set_destructor_fn(task, backend_shr_refresh_destructor);
+
+    /* Stash our task_data for use by the task thread */
+    slapi_task_set_data(task, mytaskdata);
+
+    /* start the schema reload task as a separate thread */
+    thread = PR_CreateThread(PR_USER_THREAD, backend_shr_refresh_thread,
+                             (void *)task, PR_PRIORITY_NORMAL, PR_GLOBAL_THREAD,
+                             PR_UNJOINABLE_THREAD, SLAPD_DEFAULT_THREAD_STACKSIZE);
+    if (thread == NULL) {
+        slapi_log_error(SLAPI_LOG_ERR, "nis-server",
+                        "backend_nis_refresh_task - Unable to create schema compat refresh task thread!\n");
+        *returncode = LDAP_OPERATIONS_ERROR;
+        rv = SLAPI_DSE_CALLBACK_ERROR;
+    } else {
+        /* thread successful */
+        rv = SLAPI_DSE_CALLBACK_OK;
+    }
+
+out:
+
+    return rv;
+}
 /* Scan for the list of configured domains and maps. */
 void
 backend_startup(Slapi_PBlock *pb, struct plugin_state *state)
 {
-	backend_shr_startup(state, pb, NIS_MAP_CONFIGURATION_FILTER);
+    backend_shr_startup(state, pb, NIS_MAP_CONFIGURATION_FILTER);
+    slapi_plugin_task_register_handler("NIS Server refresh task", backend_nis_refresh_task, pb);
 }
 
 void
