@@ -796,9 +796,11 @@ backend_shr_data_initialize_thread(time_t when, void *arg)
 
         if (!cbdata->state->start_priming_thread) {
             slapi_log_error(SLAPI_LOG_PLUGIN, cbdata->state->plugin_desc->spd_id,
-                "Likely a shutdown occurred before we started \n");
+                "Shutdown occurred before we started or other priming thread is already running\n");
             goto done;
         }
+        /* We are going to prim, time to prevent other priming to start */
+        cbdata->state->start_priming_thread = 0;
 
         cbdata->state->priming_tid = wrap_start_thread(&backend_shr_data_initialize_thread_cb, arg);
         if (cbdata->state->priming_tid == NULL) {
@@ -811,11 +813,134 @@ backend_shr_data_initialize_thread(time_t when, void *arg)
 				"%s tree scan will start in about %d seconds!\n",
 				cbdata->state->plugin_desc->spd_id, PLUGIN_SCAN_DELAY);
 	}
+        /* Now it is primed let others priming to proceed */
+        cbdata->state->start_priming_thread = 1;
 
 done:
         wrap_mutex_unlock(cbdata->state->priming_mutex);
 }
 
+/* routine that free the structure attached as task data */
+void
+backend_shr_refresh_destructor(Slapi_Task *task)
+{
+    if (task) {
+        task_data *mydata = (task_data *)slapi_task_get_data(task);
+        while (slapi_task_get_refcount(task) > 0) {
+            /* Yield to wait for the fixup task finishes. */
+            DS_Sleep(PR_MillisecondsToInterval(100));
+        }
+        if (mydata) {
+            /* mydata->state is global, do not free it
+             * mydata->plugin_log_id is a constant, do not free it
+             */
+            slapi_ch_free_string(&mydata->bind_dn);
+            slapi_ch_free_string(&mydata->filter);
+            /* Need to cast to avoid a compiler warning */
+            slapi_ch_free((void **)&mydata);
+        }
+    }
+}
+
+
+extern int slapi_td_set_dn(char *dn);
+
+/*
+ * Task thread
+
+ */
+void
+backend_shr_refresh_thread(void *arg)
+{
+    Slapi_Task *task = (Slapi_Task *)arg;
+    int rv = 0;
+    int total_work = 2;
+    task_data *td = NULL;
+
+    if (!task) {
+        return; /* no task */
+    }
+    slapi_task_inc_refcount(task);
+
+    /* Fetch our task data from the task */
+    td = (task_data *)slapi_task_get_data(task);
+    slapi_log_error(SLAPI_LOG_INFO, td->plugin_log_id, "refresh_thread --> refcount incremented.\n");
+
+    /* Initialize and set the bind dn in the thread data.
+     * Used for setting creatorname of the task entry.
+     */
+    slapi_td_set_dn(slapi_ch_strdup(td->bind_dn));
+
+    /* update task state to show it's running */
+    slapi_task_begin(task, total_work);
+
+    /* Check if refresh is not already running or will start soon */
+    if (td->state->ready_to_serve == 0) {
+        slapi_task_log_notice(task, "Refresh task already running or initial priming not completed yet.");
+        slapi_task_log_status(task, "Refresh task already running or initial priming not completed yet.");
+        slapi_log_error(SLAPI_LOG_INFO, td->plugin_log_id,
+                        "backend_shr_refresh_thread - Refresh task already running or initial priming not completed yet\n");
+        goto done;
+    }
+
+    slapi_task_log_notice(task, "Refresh task starts\n");
+    slapi_log_error(SLAPI_LOG_INFO, td->plugin_log_id, "backend_shr_refresh_thread - Refresh task starts\n");
+    slapi_task_inc_progress(task);
+
+    if (slapi_is_shutting_down()) {
+        slapi_task_log_notice(task, "Server is shutting down; Refresh aborted.");
+        slapi_task_log_status(task, "Server is shutting down; Refresh aborted.");
+        slapi_log_error(SLAPI_LOG_ERR, td->plugin_log_id, "backend_shr_refresh_thread - Server is shutting down; Refresh aborted.");
+    } else {
+        struct backend_shr_data_init_cbdata *cbdata = NULL;
+
+        slapi_task_log_notice(task, "Refresh maps starting soon.");
+        slapi_task_log_status(task, "Refresh maps starting soon.");
+        slapi_log_error(SLAPI_LOG_INFO, td->plugin_log_id, "backend_shr_refresh_thread - Refresh maps starting soon.\n");
+
+        cbdata = (struct backend_shr_data_init_cbdata *) slapi_ch_malloc(sizeof(struct backend_shr_data_init_cbdata));
+        if (cbdata == NULL) {
+            slapi_log_error(SLAPI_LOG_FATAL,
+                    td->plugin_log_id,
+                    "failed to create a task for populating the maps");
+            rv = SLAPI_PLUGIN_FAILURE;
+            goto done;
+        }
+        cbdata->state = td->state;
+        cbdata->filter = td->filter;
+
+        /* disable access to the maps */
+        PR_AtomicSet(&td->state->ready_to_serve, 0);
+
+        /* Free the old maps and prepare the new ones */
+        map_done(td->state);
+        map_init(NULL, td->state);
+
+        /* Schedule the initialization of the maps  */
+        slapi_eq_once(backend_shr_data_initialize_thread, cbdata, PR_SecondsToInterval(1));
+        PR_Sleep(PR_SecondsToInterval(1));
+
+        /* Then wait for its completion */
+        slapi_task_log_notice(task, "Refresh maps still going on.");
+        slapi_task_log_status(task, "Refresh maps still going on.");
+        while (td->state->ready_to_serve == 0) {
+            PR_Sleep(PR_SecondsToInterval(5));
+        }
+        slapi_task_inc_progress(task);
+
+        /* update task state to say we're finished */
+        slapi_task_log_notice(task, "Refresh maps task finished.");
+        slapi_task_log_status(task, "Refresh maps task finished.");
+        slapi_log_error(SLAPI_LOG_INFO, td->plugin_log_id, "backend_shr_refresh_thread - Refresh maps task finished.\n");
+
+    }
+done:
+
+    /* this will queue the destruction of the task */
+    slapi_task_finish(task, rv);
+    slapi_task_dec_refcount(task);
+    slapi_log_error(SLAPI_LOG_INFO, td->plugin_log_id, "backend_shr_refresh_thread <-- refcount decremented.\n");
+}
 /* Scan for the list of configured groups and sets. */
 void
 backend_shr_startup(struct plugin_state *state,
@@ -1045,10 +1170,10 @@ static bool_t
 backend_shr_entry_is_a_set(struct plugin_state *state,
 			   Slapi_PBlock *pb, Slapi_Entry *e)
 {
-	return backend_shr_entry_matches(pb, e,
-					 state->plugin_base,
-					 LDAP_SCOPE_ONELEVEL,
-					 backend_entry_get_set_config_entry_filter());
+    return backend_shr_entry_matches(pb, e,
+            state->plugin_base,
+            LDAP_SCOPE_ONELEVEL,
+            backend_entry_get_set_config_entry_filter());
 }
 
 /* Build a filter which includes the basic_filter, if given, and ANDs that
@@ -2960,6 +3085,10 @@ backend_shr_write_ignore(Slapi_PBlock *pb)
     int i = 0;
     PRBool ignore = PR_FALSE;
 
+    if (ignored_containers_sdn == NULL) {
+        return ignore;
+    }
+
     /* Check if the target DN is a subordinates of
      * on of the ignored containers
      */
