@@ -38,13 +38,15 @@ int  get_plugin_monitor_count(void);
 void set_plugin_monitor_count(int lock_count);
 void init_map_lock(void);
 extern Slapi_DN **ignored_containers_sdn;
+PRBool backend_shr_write_ignore(Slapi_PBlock *pb);
 
 
 struct plugin_state;
 
+void backend_shr_refresh_destructor(Slapi_Task *task);
+void backend_shr_refresh_thread(void *arg);
 void backend_shr_free_server_name(struct plugin_state *state, char *master);
-int backend_shr_read_server_name(Slapi_PBlock *pb, struct plugin_state *state,
-				 char **master);
+int backend_shr_read_server_name(Slapi_PBlock *pb, struct plugin_state *state, char **master);
 
 void backend_shr_free_strlist(char **strlist);
 char **backend_shr_dup_strlist_n(char **strlist, int n);
@@ -88,4 +90,5 @@ char *backend_shr_get_vattr_filter(struct plugin_state *state,
 bool_t backend_shr_get_vattr_boolean(struct plugin_state *state,
 				     Slapi_Entry *e, const char *attribute,
 				     bool_t default_value);
+const char *backend_entry_get_set_config_entry_filter();
 #endif
