@@ -1,5 +1,5 @@
 /*
- * Copyright 2008 Red Hat, Inc.
+ * Copyright 2008,2011,2013 Red Hat, Inc.
  *
  * This Program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -32,7 +32,6 @@
 #include <poll.h>
 #include <stdlib.h>
 #include <string.h>
-#include <syslog.h>
 #include <unistd.h>
 
 #ifdef HAVE_TCPD_H
@@ -48,7 +47,7 @@
 #endif
 
 #include <rpc/rpc.h>
-#include <rpcsvc/yp_prot.h>
+#include "../yp/yp.h"
 
 #include "plugin.h"
 #include "disp-nis.h"
@@ -82,7 +81,7 @@ struct dispatch_client {
 		client_replying_final,
 	} client_state;
 	/* The client's request while we're reading it. */
-	char client_inbuf[4096];
+	char client_inbuf[8192 + 4];
 	ssize_t client_inbuf_used;
 	char *client_query;
 	ssize_t client_query_size;
@@ -266,7 +265,10 @@ dispatch_securenets_check(struct plugin_state *state,
 			addr6 = client_addr->sin6.sin6_addr;
 			if ((sn->sn_family == AF_INET) &&
 			    IN6_IS_ADDR_V4MAPPED(&addr6)) {
-				if ((((uint32_t*)addr6.s6_addr)[3] &
+				memcpy(&addr.s_addr,
+				       ((uint32_t*)addr6.s6_addr) + 3,
+				       sizeof(addr.s_addr));
+				if ((addr.s_addr &
 				     sn->sn_addr.sin.netmask.s_addr) ==
 				    (sn->sn_addr.sin.address.s_addr &
 				     sn->sn_addr.sin.netmask.s_addr)) {
@@ -447,11 +449,20 @@ dispatch_dgram(struct plugin_state *state, int fd)
 	reqsize = recvfrom(cdata.dgram.client_fd, dgram, sizeof(dgram), 0,
 			   (struct sockaddr *) &cdata.dgram.client_addr,
 			   &cdata.dgram.client_addrlen);
-	slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
-			"datagram request (%d bytes)\n", reqsize);
+	if (reqsize != -1) {
+		slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
+				"datagram request (%d bytes)\n", reqsize);
+	} else {
+		slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
+				"error reading datagram request: %s\n",
+				strerror(errno));
+		free(cdata.dgram.reply_buf);
+		return;
+	}
 #ifdef HAVE_TCPD
 	/* Check tcp_wrappers access control. */
 	if ((request_set(state->request_info,
+			 RQ_FILE, -1,
 			 RQ_CLIENT_SIN, &cdata.dgram.client_addr,
 			 0) == NULL) ||
 	    (fromhost(state->request_info),
@@ -566,7 +577,13 @@ dispatch_accept_client(struct plugin_state *state, int fd)
 	}
 	linger.l_onoff = 1;
 	linger.l_linger = 0;
-	setsockopt(fd, SOL_SOCKET, SO_LINGER, &linger, sizeof(linger));
+	if (setsockopt(fd, SOL_SOCKET, SO_LINGER, &linger,
+		       sizeof(linger)) == -1) {
+		slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
+				"error marking new connected client "
+				"on %d as linger-enabled: %s\n", fd,
+				strerror(errno));
+	}
 	switch (client->client_addr.ss.ss_family) {
 	struct sockaddr_in *sin;
 	struct sockaddr_in6 *sin6;
@@ -585,6 +602,7 @@ dispatch_accept_client(struct plugin_state *state, int fd)
 #ifdef HAVE_TCPD_H
 	/* Check tcp_wrappers access control. */
 	if ((request_set(state->request_info,
+			 RQ_FILE, fd,
 			 RQ_CLIENT_SIN, &client->client_addr,
 			 0) == NULL) ||
 	    (fromhost(state->request_info),
@@ -616,7 +634,13 @@ dispatch_accept_client(struct plugin_state *state, int fd)
 	}
 	flags = fcntl(fd, F_GETFL);
 	if ((flags & O_NONBLOCK) == 0) {
-		fcntl(fd, F_SETFL, flags | O_NONBLOCK);
+		if (fcntl(fd, F_SETFL, flags | O_NONBLOCK) == -1) {
+			slapi_log_error(SLAPI_LOG_PLUGIN,
+					state->plugin_desc->spd_id,
+					"error marking new connected client "
+					"on %d as non-blocking: %s\n", fd,
+					strerror(errno));
+		}
 	}
 	slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
 			"new connected client on %d\n", fd);
@@ -667,7 +691,10 @@ client_read(struct plugin_state *state, struct dispatch_client *client)
 		     client->client_inbuf + client->client_inbuf_used,
 		     sizeof(client->client_inbuf) - client->client_inbuf_used);
 	if (count <= 0) {
-		if ((count != -1) || (errno != EAGAIN)) {
+		if ((count != -1) ||
+		    ((errno != EAGAIN) &&
+		     (errno != EWOULDBLOCK) &&
+		     (errno != EINTR))) {
 			/* Disconnect the client. */
 			if (count == 0) {
 				slapi_log_error(SLAPI_LOG_PLUGIN,
@@ -702,8 +729,9 @@ client_read(struct plugin_state *state, struct dispatch_client *client)
 		len = ntohl(nlen);
 		last = ((len & 0x80000000) != 0);
 		len &= 0x7fffffff;
-		if (len > 0x10000) {
-			/* Disconnect, because that's just ridiculous. */
+		if (len > sizeof(client->client_inbuf) - 4) {
+			/* Disconnect, because that's more data than we could
+			 * have read. */
 			slapi_log_error(SLAPI_LOG_FATAL,
 					state->plugin_desc->spd_id,
 					"client fragment claims to be %d bytes "
@@ -741,26 +769,16 @@ client_read(struct plugin_state *state, struct dispatch_client *client)
 			free(client->client_query);
 			client->client_query = query;
 			client->client_query_size = nlen;
-			/* Drop the fragment from the incoming
-			 * buffer. */
-			memmove(client->client_inbuf,
-				client->client_inbuf + (len + 4),
-				client->client_inbuf_used - (len + 4));
+			/* Drop the fragment from the incoming buffer by
+			 * moving the rest of it forward. */
+			if ((len + 4) < client->client_inbuf_used) {
+				memmove(client->client_inbuf,
+					client->client_inbuf + (len + 4),
+					client->client_inbuf_used - (len + 4));
+			}
 			client->client_inbuf_used -= (len + 4);
 		}
 		if (last) {
-			if (len > 0x10000) {
-				/* Disconnect, because that's just ridiculous.
-				 */
-				slapi_log_error(SLAPI_LOG_FATAL,
-						state->plugin_desc->spd_id,
-						"client request claims to be "
-						"%ld bytes long, assuming it's "
-						"an error\n",
-						(long) client->client_query_size);
-				client_set_closing(state, client);
-				return;
-			}
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"query is %ld bytes long on %d\n",
@@ -800,7 +818,10 @@ client_write(struct plugin_state *state, struct dispatch_client *client)
 			len, client->client_fd);
 	count = write(client->client_fd, client->client_outbuf, len);
 	if (count <= 0) {
-		if ((count != -1) || (errno != EAGAIN)) {
+		if ((count != -1) ||
+		    ((errno != EAGAIN) &&
+		     (errno != EWOULDBLOCK) &&
+		     (errno != EINTR))) {
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"error sending %ld bytes to %d\n",
@@ -867,8 +888,6 @@ dispatch_service_client(struct plugin_state *state,
 	case client_reading:
 		if (fd->revents & POLLIN) {
 			client_read(state, client);
-		} else {
-			client_set_closing(state, client);
 		}
 		break;
 	case client_replying_with_more:
@@ -876,7 +895,9 @@ dispatch_service_client(struct plugin_state *state,
 		if (fd->revents & POLLOUT) {
 			client_write(state, client);
 		} else {
-			client_set_closing(state, client);
+			if (fd->revents & (POLLERR | POLLHUP)) {
+				client_set_closing(state, client);
+			}
 		}
 		break;
 	case client_closing:
@@ -949,8 +970,8 @@ dispatch_thread(struct wrapped_thread *t)
 
 		/* Fill in the set of polling descriptors. */
 		memset(fds, 0,
-		       sizeof((state->n_listeners + client_count + 1) *
-		       sizeof(fds[0])));
+		       ((state->n_listeners + client_count + 1) *
+		        sizeof(fds[0])));
 
 		/* Add the shutdown pipe reader. */
 		fds[0].fd = wrap_thread_stopfd(t);
@@ -1001,6 +1022,7 @@ dispatch_thread(struct wrapped_thread *t)
 			slapi_log_error(SLAPI_LOG_PLUGIN,
 					state->plugin_desc->spd_id,
 					"done waiting\n");
+			free(fds);
 			return NULL;
 			break;
 		case 0:
