@@ -83,4 +83,6 @@ char **format_get_data_set(struct plugin_state *state,
 			   unsigned int **data_lengths);
 
 char *format_escape_for_filter(const char *unescaped);
+char *format_build_dn(const char *attribute, const char *value,
+		      const char *container_sdn);
 #endif
