@@ -73,6 +73,15 @@ plugin_description = {
 };
 static struct plugin_state *global_plugin_state;
 
+/* This function is call by the refresh task
+ * it is required because with the task mechanism, plugin state
+ * is not present in pblock(SLAPI_PRIVATE) so it needs to be retrieved
+ * by the plugin task, before calling the shared task
+ */
+struct plugin_state *nis_get_plugin_state()
+{
+    return global_plugin_state;
+}
 /* Populate the map cache, register with the local portmapper, and then start
  * the plugin's work thread to answer requests using the cache. */
 static int
@@ -540,6 +549,9 @@ nis_plugin_init(Slapi_PBlock *pb)
 				"error setting up plugin\n");
 		return -1;
 	}
+
+	init_map_lock();
+
 	/* Read global configuration. */
 	if ((slapi_pblock_get(pb, SLAPI_PLUGIN_CONFIG_ENTRY,
 			      &plugin_entry) == 0) &&
@@ -595,7 +607,13 @@ nis_plugin_init(Slapi_PBlock *pb)
 #endif
 	slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
 			"registered plugin hooks\n");
-	global_plugin_state = NULL;
+	/* The registered plugin callbacks retrieve the global_plugin_state
+	 * from the pblock(SLAPI_PLUGIN_PRIVATE).
+	 * The task mechanism does not provide the reference to global_plugin_state.
+	 * So for the map refresh task, global_plugin_state should not be reset
+	 * and it is provided by nis_get_plugin_state().
+	 * global_plugin_state = NULL;
+	 */
 	/* Note that the plugin was successfully loaded. */
 	slapi_log_error(SLAPI_LOG_PLUGIN, plugin_description.spd_id,
 			"plugin initialized\n");
