@@ -79,6 +79,15 @@ plugin_description = {
 };
 static struct plugin_state *global_plugin_state;
 
+/* This function is call by the refresh task
+ * it is required because with the task mechanism, plugin state
+ * is not present in pblock(SLAPI_PRIVATE) so it needs to be retrieved
+ * by the plugin task, before calling the shared task
+ */
+struct plugin_state *schema_compat_get_plugin_state()
+{
+    return global_plugin_state;
+}
 /* Handle the part of startup that needs to be done before we drop privileges,
  * which for this plugin isn't much at all. */
 static int
@@ -441,7 +450,13 @@ schema_compat_plugin_init(Slapi_PBlock *pb)
 	}
 	slapi_log_error(SLAPI_LOG_PLUGIN, state->plugin_desc->spd_id,
 			"registered plugin hooks\n");
-	global_plugin_state = NULL;
+	/* The registered plugin callbacks retrieve the global_plugin_state
+	 * from the pblock(SLAPI_PLUGIN_PRIVATE).
+	 * The task mechanism does not provide the reference to global_plugin_state.
+	 * So for the map refresh task, global_plugin_state should not be reset
+	 * and it is provided by schema_compat_get_plugin_state().
+	 * global_plugin_state = NULL;
+	 */
 	/* Note that the plugin was successfully loaded. */
 	slapi_log_error(SLAPI_LOG_PLUGIN, plugin_description.spd_id,
 			"plugin initialized\n");
